@@ -3,13 +3,15 @@
 #
 #  (C) Paul Evans, 2021-2023 -- leonerd@leonerd.org.uk
 
-package Syntax::Operator::Zip 0.06;
+package Syntax::Operator::Zip 0.07;
 
 use v5.14;
 use warnings;
 
 use Carp;
 
+use meta;
+
 require XSLoader;
 XSLoader::load( __PACKAGE__, our $VERSION );
 
@@ -96,12 +98,15 @@ sub apply
           : delete $^H{"Syntax::Operator::Zip/$_"};
    }
 
+   my $callerpkg;
+
    foreach (qw( zip mesh )) {
       next unless delete $syms{$_};
 
-      no strict 'refs';
-      $on ? *{"${caller}::$_"} = \&{$_}
-          : warn "TODO: implement unimport of package symbol";
+      $callerpkg //= meta::get_package( $caller );
+
+      $on ? $callerpkg->add_symbol( '&'.$_ => \&{$_} )
+          : $callerpkg->remove_symbol( '&'.$_ );
    }
 
    croak "Unrecognised import symbols @{[ keys %syms ]}" if keys %syms;
